package sun.security.krb5.internal.ccache;

import sun.security.krb5.internal.Krb5;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import sun.security.action.GetPropertyAction;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.nio.charset.StandardCharsets;
import sun.security.util.SecurityProperties;
import java.util.Collections;
import sun.security.krb5.internal.LoginOptions;
import sun.security.krb5.Asn1Exception;
import java.util.Iterator;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import sun.security.krb5.KrbException;
import java.io.IOException;
import java.io.File;
import java.util.List;
import java.util.Vector;
import sun.security.krb5.PrincipalName;

public class FileCredentialsCache extends CredentialsCache implements FileCCacheConstants
{
    public int version;
    public Tag tag;
    public PrincipalName primaryPrincipal;
    private Vector<Credentials> credentialsList;
    private static String dir;
    private static boolean DEBUG;
    private List<CredentialsCache.ConfigEntry> configEntries;
    
    public static synchronized FileCredentialsCache acquireInstance(PrincipalName primaryPrincipal, String s) {
        try {
            FileCredentialsCache fileCredentialsCache = new FileCredentialsCache();
            if (s == null) {
                FileCredentialsCache.cacheName = getDefaultCacheName();
            }
            else {
                FileCredentialsCache.cacheName = checkValidation(s);
            }
            if (FileCredentialsCache.cacheName == null || !new File(FileCredentialsCache.cacheName).exists()) {
                return null;
            }
            if (primaryPrincipal != null) {
                fileCredentialsCache.primaryPrincipal = primaryPrincipal;
            }
            fileCredentialsCache.load(FileCredentialsCache.cacheName);
            return fileCredentialsCache;
        }
        catch (IOException ex) {
            if (FileCredentialsCache.DEBUG) {
                ex.printStackTrace();
            }
        }
        catch (KrbException ex2) {
            if (FileCredentialsCache.DEBUG) {
                ex2.printStackTrace();
            }
        }
        return null;
    }
    
    public static FileCredentialsCache acquireInstance() {
        return acquireInstance(null, null);
    }
    
    static synchronized FileCredentialsCache New(PrincipalName principalName, String s) {
        try {
            FileCredentialsCache fileCredentialsCache = new FileCredentialsCache();
            FileCredentialsCache.cacheName = checkValidation(s);
            if (FileCredentialsCache.cacheName == null) {
                return null;
            }
            fileCredentialsCache.init(principalName, FileCredentialsCache.cacheName);
            return fileCredentialsCache;
        }
        catch (IOException ex) {}
        catch (KrbException ex2) {}
        return null;
    }
    
    static synchronized FileCredentialsCache New(PrincipalName principalName) {
        try {
            FileCredentialsCache fileCredentialsCache = new FileCredentialsCache();
            fileCredentialsCache.init(principalName, FileCredentialsCache.cacheName = getDefaultCacheName());
            return fileCredentialsCache;
        }
        catch (IOException ex) {
            if (FileCredentialsCache.DEBUG) {
                ex.printStackTrace();
            }
        }
        catch (KrbException ex2) {
            if (FileCredentialsCache.DEBUG) {
                ex2.printStackTrace();
            }
        }
        return null;
    }
    
    private FileCredentialsCache() {
        this.configEntries = new ArrayList();
    }
    
    boolean exists(String pathname) {
        return new File(pathname).exists();
    }
    
    synchronized void init(PrincipalName primaryPrincipal, String name) throws IOException, KrbException {
        this.primaryPrincipal = primaryPrincipal;
        try (final FileOutputStream fileOutputStream = new FileOutputStream(name);
             final CCacheOutputStream cCacheOutputStream = new CCacheOutputStream(fileOutputStream)) {
            this.version = 1283;
            cCacheOutputStream.writeHeader(this.primaryPrincipal, this.version);
        }
        this.load(name);
    }
    
    synchronized void load(String name) throws IOException, KrbException {
        try (final FileInputStream fileInputStream = new FileInputStream(name);
             final CCacheInputStream cCacheInputStream = new CCacheInputStream(fileInputStream)) {
            this.version = cCacheInputStream.readVersion();
            if (this.version == 1284) {
                this.tag = cCacheInputStream.readTag();
            }
            else {
                this.tag = null;
                if (this.version == 1281 || this.version == 1282) {
                    cCacheInputStream.setNativeByteOrder();
                }
            }
            PrincipalName principal = cCacheInputStream.readPrincipal(this.version);
            if (this.primaryPrincipal != null) {
                if (!this.primaryPrincipal.match(principal)) {
                    throw new IOException("Primary principals don't match.");
                }
            }
            else {
                this.primaryPrincipal = principal;
            }
            this.credentialsList = new Vector();
            while (cCacheInputStream.available() > 0) {
                Object cred = cCacheInputStream.readCred(this.version);
                if (cred != null) {
                    if ((cred instanceof Credentials)) {
                        this.credentialsList.addElement((Credentials)cred);
                    }
                    else {
                        this.addConfigEntry((CredentialsCache.ConfigEntry)cred);
                    }
                }
                else {
                    continue;
                }
            }
        }
    }
    
    @Override
    public synchronized void update(Credentials obj) {
        if (this.credentialsList != null) {
            if (this.credentialsList.isEmpty()) {
                this.credentialsList.addElement(obj);
            }
            else {
                boolean n = false;
                for (int i = 0; i < this.credentialsList.size(); ++i) {
                    Credentials credentials = (Credentials)this.credentialsList.elementAt(i);
                    if (this.match(obj.sname.getNameStrings(), credentials.sname.getNameStrings()) && obj.sname.getRealmString().equalsIgnoreCase(credentials.sname.getRealmString())) {
                        n = true;
                        if (obj.endtime.getTime() >= credentials.endtime.getTime()) {
                            if (FileCredentialsCache.DEBUG) {
                                System.out.println(" >>> FileCredentialsCache Ticket matched, overwrite the old one.");
                            }
                            this.credentialsList.removeElementAt(i);
                            this.credentialsList.addElement(obj);
                        }
                    }
                }
                if (n == 0) {
                    if (FileCredentialsCache.DEBUG) {
                        System.out.println(" >>> FileCredentialsCache Ticket not exactly matched, add new one into cache.");
                    }
                    this.credentialsList.addElement(obj);
                }
            }
        }
    }
    
    @Override
    public synchronized PrincipalName getPrimaryPrincipal() {
        return this.primaryPrincipal;
    }
    
    @Override
    public synchronized void save() throws IOException, Asn1Exception {
        try (final FileOutputStream fileOutputStream = new FileOutputStream(FileCredentialsCache.cacheName);
             final CCacheOutputStream cCacheOutputStream = new CCacheOutputStream(fileOutputStream)) {
            cCacheOutputStream.writeHeader(this.primaryPrincipal, this.version);
            Credentials[] credsList;
            if ((credsList = this.getCredsList()) != null) {
                for (int i = 0; i < credsList.length; ++i) {
                    cCacheOutputStream.addCreds(credsList[i]);
                }
            }
            Iterator<CredentialsCache.ConfigEntry> iterator = this.getConfigEntries().iterator();
            while (iterator.hasNext()) {
                cCacheOutputStream.addConfigEntry(this.primaryPrincipal, (CredentialsCache.ConfigEntry)iterator.next());
            }
        }
    }
    
    boolean match(String[] array, String[] array2) {
        if (array.length != array2.length) {
            return false;
        }
        int i = 0;
        while (i < array.length) {
            if (!array[i].equalsIgnoreCase(array2[i])) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    @Override
    public synchronized Credentials[] getCredsList() {
        if (this.credentialsList == null || this.credentialsList.isEmpty()) {
            return null;
        }
        Credentials[] array = new Credentials[this.credentialsList.size()];
        for (int i = 0; i < this.credentialsList.size(); ++i) {
            array[i] = (Credentials)this.credentialsList.elementAt(i);
        }
        return array;
    }
    
    @Override
    public Credentials getCreds(LoginOptions options, PrincipalName principalName) {
        if (options == null) {
            return this.getCreds(principalName);
        }
        Credentials[] credsList = this.getCredsList();
        if (credsList == null) {
            return null;
        }
        int i = 0;
        while (i < credsList.length) {
            if (principalName.match(credsList[i].sname) && credsList[i].flags.match(options)) {
                return credsList[i];
            }
            ++i;
        }
        return null;
    }
    
    @Override
    public void addConfigEntry(CredentialsCache.ConfigEntry configEntry) {
        this.configEntries.add(configEntry);
    }
    
    @Override
    public List<CredentialsCache.ConfigEntry> getConfigEntries() {
        return Collections.unmodifiableList(this.configEntries);
    }
    
    @Override
    public Credentials getCreds(PrincipalName principalName) {
        Credentials[] credsList = this.getCredsList();
        if (credsList == null) {
            return null;
        }
        int i = 0;
        while (i < credsList.length) {
            if (principalName.match(credsList[i].sname)) {
                return credsList[i];
            }
            ++i;
        }
        return null;
    }
    
    @Override
    public sun.security.krb5.Credentials getInitialCreds() {
        Credentials defaultCreds = this.getDefaultCreds();
        if (defaultCreds == null) {
            return null;
        }
        sun.security.krb5.Credentials setKrbCreds = defaultCreds.setKrbCreds();
        CredentialsCache.ConfigEntry configEntry = this.getConfigEntry("proxy_impersonator");
        if (configEntry == null) {
            if (FileCredentialsCache.DEBUG) {
                System.out.println("get normal credential");
                return setKrbCreds;
            }
            return setKrbCreds;
        }
        else {
            String privilegedGetOverridable = SecurityProperties.privilegedGetOverridable("jdk.security.krb5.default.initiate.credential");
            if (privilegedGetOverridable == null) {
                privilegedGetOverridable = "always-impersonate";
            }
            int n = -1;
            switch (privilegedGetOverridable.hashCode()) {
                case 440640621: {
                    if (privilegedGetOverridable.equals("no-impersonate")) {
                        n = 0;
                        break;
                    }
                    break;
                }
                case -1438305945: {
                    if (privilegedGetOverridable.equals("try-impersonate")) {
                        n = 1;
                        break;
                    }
                    break;
                }
                case -222747717: {
                    if (privilegedGetOverridable.equals("always-impersonate")) {
                        n = 2;
                        break;
                    }
                    break;
                }
            }
            int n2;
            switch (n) {
                case 0: {
                    if (FileCredentialsCache.DEBUG) {
                        System.out.println("get normal credential");
                        return setKrbCreds;
                    }
                    return setKrbCreds;
                }
                case 1: {
                    n2 = false;
                    break;
                }
                case 2: {
                    n2 = true;
                    break;
                }
                default: {
                    throw new RuntimeException("Invalid jdk.security.krb5.default.initiate.credential");
                }
            }
            try {
                PrincipalName o = new PrincipalName(new String(configEntry.getData(), StandardCharsets.UTF_8));
                if (!setKrbCreds.getClient().equals(o)) {
                    if (FileCredentialsCache.DEBUG) {
                        System.out.println("proxy_impersonator does not match service name");
                    }
                    return n2 ? null : setKrbCreds;
                }
                else {
                    PrincipalName primaryPrincipal = this.getPrimaryPrincipal();
                    Credentials credentials = null;
                    Credentials[] credsList = this.getCredsList();
                    int length = credsList.length;
                    int i = 0;
                    while (i < length) {
                        Credentials credentials2 = credsList[i];
                        if (credentials2.getClientPrincipal().equals(primaryPrincipal) && credentials2.getServicePrincipal().equals(o)) {
                            credentials = credentials2;
                            break;
                        }
                        ++i;
                    }
                    if (credentials == null) {
                        if (FileCredentialsCache.DEBUG) {
                            System.out.println("Cannot find evidence ticket in ccache");
                        }
                        return n2 ? null : setKrbCreds;
                    }
                    else {
                        if (FileCredentialsCache.DEBUG) {
                            System.out.println("Get proxied credential");
                        }
                        return setKrbCreds.setProxy(credentials.setKrbCreds());
                    }
                }
            }
            catch (KrbException ex) {
                if (FileCredentialsCache.DEBUG) {
                    System.out.println("Impersonation with ccache failed");
                }
                return n2 ? null : setKrbCreds;
            }
        }
    }
    
    @Override
    public Credentials getDefaultCreds() {
        Credentials[] credsList = this.getCredsList();
        if (credsList == null) {
            return null;
        }
        int i = credsList.length - 1;
        while (i >= 0) {
            if (credsList[i].sname.toString().startsWith("krbtgt") && credsList[i].sname.getNameStrings()[1].equals(credsList[i].sname.getRealm().toString())) {
                return credsList[i];
            }
            --i;
        }
        return null;
    }
    
    public static String getDefaultCacheName() {
        String str = "krb5cc";
        String str2 = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                String s = System.getenv("KRB5CCNAME");
                if (s != null && s.length() >= 5 && s.regionMatches(true, 0, "FILE:", 0, 5)) {
                    s = s.substring(5);
                    return s;
                }
                return s;
            }
        });
        if (str2 != null) {
            if (FileCredentialsCache.DEBUG) {
                System.out.println(">>>KinitOptions cache name is " + str2);
                return str2;
            }
            return str2;
        }
        else {
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("os.name"));
            if (s != null && !s.startsWith("Windows")) {
                try {
                    Class<Object> forName = Class.forName("com.sun.security.auth.module.UnixSystem");
                    String string = File.separator + "tmp" + File.separator + str + "_" + (long)forName.getMethod("getUid", new Class[0]).invoke(forName.getConstructor(new Class[0]).newInstance(new Object[0]), new Object[0]);
                    if (FileCredentialsCache.DEBUG) {
                        System.out.println(">>>KinitOptions cache name is " + string);
                        return string;
                    }
                    return string;
                }
                catch (Exception ex) {
                    if (FileCredentialsCache.DEBUG) {
                        System.out.println("Exception in obtaining uid for Unix platforms Using user's home directory");
                        ex.printStackTrace();
                    }
                }
            }
            String str3 = (String)AccessController.doPrivileged(new GetPropertyAction("user.name"));
            String s2 = (String)AccessController.doPrivileged(new GetPropertyAction("user.home"));
            if (s2 == null) {
                s2 = (String)AccessController.doPrivileged(new GetPropertyAction("user.dir"));
            }
            String str4;
            if (str3 != null) {
                str4 = s2 + File.separator + str + "_" + str3;
            }
            else {
                str4 = s2 + File.separator + str;
            }
            if (FileCredentialsCache.DEBUG) {
                System.out.println(">>>KinitOptions cache name is " + str4);
                return str4;
            }
            return str4;
        }
    }
    
    public static String checkValidation(String pathname) {
        if (pathname == null) {
            return null;
        }
        String canonicalPath;
        try {
            canonicalPath = new File(pathname).getCanonicalPath();
            File file = new File(canonicalPath);
            if (!file.exists() && !new File(file.getParent()).isDirectory()) {
                canonicalPath = null;
                goto Label_0057;
            }
        }
        catch (IOException ex) {
            canonicalPath = null;
        }
        return canonicalPath;
    }
    
    private static String exec(String str) {
        StringTokenizer stringTokenizer = new StringTokenizer(str);
        Vector<String> vector = new Vector();
        while (stringTokenizer.hasMoreTokens()) {
            vector.addElement(stringTokenizer.nextToken());
        }
        final String[] anArray = new String[vector.size()];
        vector.copyInto(anArray);
        try {
            Process process = (Process)AccessController.doPrivileged(new PrivilegedAction<Process>() {
                @Override
                public Process run() {
                    try {
                        return Runtime.getRuntime().exec(anArray);
                    }
                    catch (IOException ex) {
                        if (FileCredentialsCache.DEBUG) {
                            ex.printStackTrace();
                        }
                        return null;
                    }
                }
            });
            if (process == null) {
                return null;
            }
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(process.getInputStream(), "8859_1"));
            String s;
            if (anArray.length == 1 && anArray[0].equals("/usr/bin/env")) {
                while ((s = bufferedReader.readLine()) != null) {
                    if (s.length() < 11 || !s.substring(0, 11).equalsIgnoreCase("KRB5CCNAME=")) {
                        continue;
                    }
                    s = s.substring(11);
                    break;
                }
            }
            else {
                s = bufferedReader.readLine();
            }
            bufferedReader.close();
            return s;
        }
        catch (Exception ex) {
            if (FileCredentialsCache.DEBUG) {
                ex.printStackTrace();
            }
            return null;
        }
    }
    
    static {
        FileCredentialsCache.DEBUG = Krb5.DEBUG;
    }
}
